;****************************************************************
;   DFT module for C language
;   Discrete Fourier Transform
;   Digital Signal Processing modules
;   Copyright(c) Bognr Andrs   -  Quenne Electric 09.2006
;****************************************************************

;16 bit MONO input
;1024 dot DFT
;Fourier module (fourier.asm) is 256 dot DFT

[BITS 16]		;Set code generation to 16 bit mode
[SEGMENT .text]		;Main code segment

GLOBAL _dft1024         ;DFT C function
GLOBAL _idft1024        ;Inverse DFT function

EXTERN _hk              ;int hk[1024] input buffer
EXTERN _amp             ;float amp[1024] output buffer - amplitude
EXTERN _fi              ;float fi[1024] output buffer - fi
EXTERN _re              ;float re[1024] output buffer -Re{f}
EXTERN _im              ;float im[1024] output buffer -Im{f}

cpy db 'Copyright(c) Bognr Andrs 2006',0
k dw 0             ;ciklusvltoz
n dw 0             ;ciklusvltoz
Ndot dw 1024       ;N=a pontok szma
const dd 0         ;const=2*pi/Ndot

_dft1024:          ;Discrete Fourier Transform
push ax
push bx
push cx
finit
mov cx,1024
xor ax,ax
mov [cs:n],ax
cikl2: push cx
xor ax,ax
mov [cs:k],ax
mov cx,1024
fldz               ;szumma gyjthelye
fldz
cikl1:
fldpi
fldpi
faddp st1,st0      ;st(0)=2*pi
fimul word [cs:k]
fimul word [cs:n]
fidiv word [cs:Ndot]
fsincos
mov bx,[cs:k]
shl bx,1           ;*2
fimul word [bx+_hk];szorzs hk-val
fild word [bx+_hk]
fmulp st2
faddp st2,st0      ;szumma
faddp st2,st0
inc word [cs:k]
loop cikl1
fchs
mov bx,[cs:n]
shl bx,2
fstp dword [bx+_re]
fchs
fstp dword [bx+_im]
inc word [cs:n]
pop cx
loop cikl2

;A(f)=sqrt(Re(f)2+Im(f)2)
;Fi(f)=arctg(Im(f)/Re(f))

;A(f)kiszmtsa
mov cx,1024
xor bx,bx          ;array vltoz
cikl3:
fld dword [bx+_re]
fmul dword [bx+_re]
fld dword [bx+_im]
fmul dword [bx+_im]
faddp st1,st0
fsqrt
fstp dword [bx+_amp]
add bx,4
loop cikl3

;Fi(f)kiszmtsa
mov cx,1024
xor bx,bx          ;array vltoz
cikl4:
fld dword [bx+_im]
fdiv dword [bx+_re]
fld1
fpatan
fstp dword [bx+_fi]
add bx,4
loop cikl4

pop cx
pop bx
pop ax
retf




_idft1024:         ;Inverse Discrete Fourier Transform
push ax
push bx
push cx
finit
xor ax,ax
mov [cs:k],ax
fldpi                   ;const=2*pi/Ndot
fldpi
faddp st1,st0
fidiv word [cs:Ndot]
fstp dword [cs:const]
mov cx,1024
icikl2:
push cx
xor ax,ax
mov [cs:n],ax
mov cx,1024
fldz                    ;szumma gyjthely
fldz
icikl1:
fld dword [cs:const]
fimul word [cs:n]
fimul word [cs:k]
mov bx,[cs:n]
shl bx,2
fadd dword [bx+_fi]
fsincos
fmul dword [bx+_amp]
fld dword [bx+_amp]
fmulp st2
faddp st2,st0
faddp st2,st0
inc word [cs:n]
loop icikl1
fidiv word [cs:Ndot]    ;*1/N
fild word [cs:Ndot]
fdivp st2,st0
faddp st1,st0
mov bx,[cs:k]
shl bx,1
fistp word [bx+_hk]
inc word [cs:k]
pop cx
loop icikl2
pop cx
pop bx
pop ax
retf